package cn.javacoder.test_fastdfs;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import org.csource.common.NameValuePair;
import org.csource.fastdfs.ClientGlobal;
import org.csource.fastdfs.StorageClient;
import org.csource.fastdfs.StorageServer;
import org.csource.fastdfs.TrackerClient;
import org.csource.fastdfs.TrackerServer;

public class MultiThreadUpload {

	public static void main(String[] args) throws Exception {
		String configFileName = "src/main/resources/fdfs_client.conf";
		ClientGlobal.init(configFileName);
		uploadFiles();
	}
	public static void uploadFiles() throws Exception {
		String dir = "D:\\Ted\\CPS";
		File root = new File(dir);
		List<File> dirList = new ArrayList<File>();
		dirList.add(root);
		
		ExecutorService executor = Executors.newFixedThreadPool(5);
		long begin = System.currentTimeMillis();
		int count = 0;
		while(!dirList.isEmpty()) {
			File currentDir = dirList.remove(0);
			File[] child = currentDir.listFiles();
			for(File f : child) {
				if(f.isDirectory()) {
					dirList.add(f);
				} else if(f.isFile() && f.getName().endsWith(".java")) {
					executor.execute(new UploadRunnable(f));
					count ++;
				}
			}
		}
		while(count > UploadRunnable.count + 5) {
			Thread.sleep(2000);
			System.out.println("--------sleep 2 seconds count " + count + " processed:" + UploadRunnable.count);
		}
		executor.shutdown();
		executor.awaitTermination(5, TimeUnit.SECONDS);
		System.out.println("total processed:" + count + " , elapsed:" + ((System.currentTimeMillis() - begin)/1000 ));
	}
	
	public static class UploadRunnable implements Runnable {
		public static int count = 0;
		private File f;
		public UploadRunnable(File f) {
			this.f = f;
		}
		@Override
		public void run() {
			String[] fileRemoteInfo = null;
			try {
				//连接对象不能重用
				TrackerClient tracker = new TrackerClient();
				TrackerServer trackerServer = tracker.getConnection();
				if(trackerServer != null) {
					StorageServer storageServer = null;
					StorageClient client = new StorageClient(trackerServer, storageServer);
					fileRemoteInfo = uploadFile(client, f, f.getName(), f.length());
					trackerServer.close();
				}
			} catch (IOException e) {
				e.printStackTrace();
			} finally {
				synchronized (UploadRunnable.class) {
					count ++;
				}
			}
			if(fileRemoteInfo != null) {
				System.out.println(Arrays.asList(fileRemoteInfo));
			}
			
		}
		
	}
	
	/**
	 * 上传文件
	 */
	public static String[] uploadFile(StorageClient client, File file, String uploadFileName, long fileLength)
			throws IOException {
		byte[] fileBuff = getFileBuffer(new FileInputStream(file), fileLength);
		String[] files = null;
		String fileExtName = "";
		if (uploadFileName.contains(".")) {
			fileExtName = uploadFileName.substring(uploadFileName.lastIndexOf(".") + 1);
		} else {
			System.out.println("Fail to upload file, because the format of filename is illegal.");
			return null;
		}

		// 设置元信息
		NameValuePair[] metaList = new NameValuePair[3];
		metaList[0] = new NameValuePair("fileName", uploadFileName);
		metaList[1] = new NameValuePair("fileExtName", fileExtName);
		metaList[2] = new NameValuePair("fileLength", String.valueOf(fileLength));

		// 上传文件
		try {
			files = client.upload_file(fileBuff, fileExtName, metaList);
		} catch (Exception e) {
			System.out.println("Upload file \"" + uploadFileName + "\"fails");
		}
		return files;
	}
	
	private static byte[] getFileBuffer(InputStream inStream, long fileLength) throws IOException {

		byte[] buffer = new byte[256 * 1024];
		byte[] fileBuffer = new byte[(int) fileLength];

		int count = 0;
		int length = 0;

		while ((length = inStream.read(buffer)) != -1) {
			for (int i = 0; i < length; ++i) {
				fileBuffer[count + i] = buffer[i];
			}
			count += length;
		}
		inStream.close();
		return fileBuffer;
	}
}
