package cn.javacoder.test_fastdfs;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;

import org.csource.common.NameValuePair;
import org.csource.fastdfs.ClientGlobal;
import org.csource.fastdfs.StorageClient;
import org.csource.fastdfs.StorageServer;
import org.csource.fastdfs.TrackerClient;
import org.csource.fastdfs.TrackerServer;

/**
 * Hello world!
 *
 */
public class SimpleTest {
	public static void main(String[] args) throws Exception {
		String configFileName = "src/main/resources/fdfs_client.conf";
		ClientGlobal.init(configFileName);
		TrackerClient tracker = new TrackerClient();
		TrackerServer trackerServer = tracker.getConnection();
		StorageServer storageServer = null;
		StorageClient client = new StorageClient(trackerServer, storageServer);
		
		//上传示例
		String[] fileNames = {"aliyun120.jpg",
				"jiandaoyun120.jpg",
				"sspaas320.jpg",
				"sspaas700.jpg",
				"submail120.jpg"};
		for(String fileName: fileNames) {
			File file = new File("src/main/resources/images/" + fileName);
			String[] files = uploadFile(client, file, fileName, file.length());
			System.out.println(Arrays.asList(files));
		}
		
		//下载示例
		//download(client);
	}
	
	public static void download(StorageClient client) throws Exception {
		String groupName = "group3";
		String remote_filename =  "M00/01/74/wKgOmVh_OT2ASButAAABEXunzsI56.conf";
		byte[] fileBytes = client.download_file(groupName, remote_filename);
		FileOutputStream f = new FileOutputStream("D:\\Ted\\TEST\\test_fastdfs\\src\\main\\resources\\downloaded.conf");
		f.write(fileBytes);
		f.close();
	}
	

	
	
	/**
	 * 上传文件
	 */
	public static String[] uploadFile(StorageClient client,File file, String uploadFileName, long fileLength)
			throws IOException {
		byte[] fileBuff = getFileBuffer(new FileInputStream(file), fileLength);
		String[] files = null;
		String fileExtName = "";
		if (uploadFileName.contains(".")) {
			fileExtName = uploadFileName.substring(uploadFileName.lastIndexOf(".") + 1);
		} else {
			System.out.println("Fail to upload file, because the format of filename is illegal.");
			return null;
		}

		// 设置元信息
		NameValuePair[] metaList = new NameValuePair[3];
		metaList[0] = new NameValuePair("fileName", uploadFileName);
		metaList[1] = new NameValuePair("fileExtName", fileExtName);
		metaList[2] = new NameValuePair("fileLength", String.valueOf(fileLength));

		// 上传文件
		try {
			files = client.upload_file(fileBuff, fileExtName, metaList);
		} catch (Exception e) {
			System.out.println("Upload file \"" + uploadFileName + "\"fails");
		}
		return files;
	}

	private static byte[] getFileBuffer(InputStream inStream, long fileLength) throws IOException {

		byte[] buffer = new byte[256 * 1024];
		byte[] fileBuffer = new byte[(int) fileLength];

		int count = 0;
		int length = 0;

		while ((length = inStream.read(buffer)) != -1) {
			for (int i = 0; i < length; ++i) {
				fileBuffer[count + i] = buffer[i];
			}
			count += length;
		}
		inStream.close();
		return fileBuffer;
	}

}
